package gdrivemount

import (
	"fmt"
	"os"
	"strconv"
	"strings"
	"time"

	"github.com/jacobsa/fuse/fuseops"
	drive "google.golang.org/api/drive/v3"
)

type FileEntry struct {
	fs    *GDriveFS
	GFile *drive.File
	isDir bool
	Name  string // local name
	Inode fuseops.InodeID
	Attr  fuseops.InodeAttributes
	// inode?
	//fileMap map[string]*FileEntry
	fileList []*FileEntry // children
	//fileMap map[string]*FileEntry // Children?
}

// Load append whatever?
// append file to this tree
func (fe *FileEntry) appendGFile(f *drive.File) *FileEntry {

	name := f.Name
	count := 1
	nameParts := strings.Split(f.Name, ".")
	for {
		en := fe.findByName(name, false) // locally only
		if en == nil {                   // ok we want no value
			break
		}
		count++
		if len(nameParts) > 1 {
			name = fmt.Sprintf("%s(%d).%s", nameParts[0], count, strings.Join(nameParts[1:], "."))
		} else {
			name = fmt.Sprintf("%s(%d)", nameParts[0], count)
		}
	}

	uid, _ := strconv.Atoi(fe.fs.osuser.Uid)
	gid, _ := strconv.Atoi(fe.fs.osuser.Gid)

	// Create Attribute
	attr := fuseops.InodeAttributes{}
	attr.Nlink = 1
	attr.Size = uint64(f.Size)
	// Temp
	attr.Uid = uint32(uid)
	attr.Gid = uint32(gid)

	attr.Crtime, _ = time.Parse(time.RFC3339, f.CreatedTime)
	attr.Mtime, _ = time.Parse(time.RFC3339, f.ModifiedTime)
	attr.Mode = os.FileMode(0644) // default
	if f.MimeType == "application/vnd.google-apps.folder" {
		attr.Mode = os.FileMode(0755) | os.ModeDir
	}
	// Create an entry
	entry := &FileEntry{
		fs:    fe.fs,
		GFile: f,
		Name:  name,                    // we dont need name
		Inode: fe.fs.findUnusedInode(), // Lock somehow
		Attr:  attr,
		//fileMap: map[string]*FileEntry{},
		fileList: []*FileEntry{},
	}

	fe.fileList = append(fe.fileList, entry)
	//fe.fileMap[f.Name] = entry

	return entry
}
func (fe *FileEntry) findByInode(inode fuseops.InodeID, recurse bool) *FileEntry {
	if inode == fe.Inode {
		return fe // return self
	}
	// Recurse??
	for _, e := range fe.fileList {
		if e.Inode == inode {
			return e
		}
		if recurse {
			re := e.findByInode(inode, recurse)
			if re != nil {
				return re
			}
		}
	}
	// For each child we findByInode
	return nil
}

func (fe *FileEntry) findByName(name string, recurse bool) *FileEntry {
	// Recurse??
	for _, e := range fe.fileList {
		if e.Name == name {
			return e
		}
		if recurse {
			re := e.findByName(name, recurse)
			if re != nil {
				return re
			}
		}
	}
	// For each child we findByInode
	return nil
}

func (fe *FileEntry) findByGID(gdriveID string, recurse bool) *FileEntry {
	// Recurse??
	for _, e := range fe.fileList {
		if e.GFile.Id == gdriveID {
			return e
		}
		if recurse {
			re := e.findByGID(gdriveID, recurse)
			if re != nil {
				return re
			}
		}
	}
	// For each child we findByInode
	return nil
}
